# -*- coding: cp1251 -*-
__author__ = "rual"
#      X-PLANE(10)

import math
import numpy
import struct
from math import cos, sin, radians, degrees, atan2, sqrt

ComPort = 'COM15'    #    
BaudRate = 115200   #  

rx_port = 49005 
tx_port = 49000

def ang2q(ax,ay,az):
    sin_x =sin(ax*0.5);
    cos_x =cos(ax*0.5);
    sin_y =sin(ay*0.5);
    cos_y =cos(ay*0.5);
    sin_z =sin(az*0.5);
    cos_z =cos(az*0.5);
		
    x = cos_x*sin_y*sin_z+sin_x*cos_y*cos_z;
    y =	cos_x*sin_y*cos_z-sin_x*cos_y*sin_z;
    z = cos_x*cos_y*sin_z+sin_x*sin_y*cos_z;
    w = cos_x*cos_y*cos_z-sin_x*sin_y*sin_z;
    return (x,y,z,w)

def vect2board(vx,vy,vz,(x,y,z,w)):
    qx = vx * w + vz * y - vy * z
    qy = vy * w + vx * z - vz * x
    qz = vz * w + vy * x - vx * y
    qw = vx * x + vy * y + vz * z

    rx = w * qx + x * qw + y * qz - z * qy
    ry = w * qy + y * qw + z * qx - x * qz
    rz = w * qz + z * qw + x * qy - y * qx
    return (rx,ry,rz)


###################    -  ##########################
import socket
class XPlane:    
    host = '127.0.0.1'    
    tx_port = 49000
    xplane_in_socket  = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    xplane_out_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)    
    channels = [0]*132
    def __init__(self,rx_p): # 
        self.bind(rx_p)        
        for i in range(132): #     
            self.channels[i] = [0.0]*8
    def __del__(self): # 
        self.close()
        
    my_hostname =''
    my_host = '127.0.0.1'
    rx_port = 49005
    def bind(self, rx_port):
        self.my_hostname = socket.gethostname()
        self.my_host = socket.gethostbyname(self.my_hostname)
        print self.my_host
            
        try:
            self.xplane_in_socket.bind((self.my_host,self.rx_port))
        except Exception, e1:
            print '...xp error bind:',e1
            pass
        
    def connect(self, host, tx_port):
        self.host = host
        try:
            self.xplane_out_socket.connect((host,tx_port))
        except Exception, e1:
            print '...xp error connect:',e1
            pass
        
    def rx_pack(self):        
        try: #              
            data,addr = self.xplane_in_socket.recvfrom(4096)
            return  data,addr
        except IOError as self.e:
            print self.e.errno,':',self.e.message
            self.close()
            pass 
    def tx_pack(self, data):
        try: #  
            data = 'DATA@' + data
            #print data
            num =0
            num = self.xplane_in_socket.sendto(data,(self.host, self.tx_port))
            return num
        except IOError as self.e:
            print self.e.errno,':',self.e.message
            self.close()
            pass
    def close(self):
        try:
            self.xplane_in_socket.close()
        except IOError as self.e:
            print self.e.errno,':',self.e.message
        try:
            self.xplane_out_socket.close()
        except IOError as self.e:
            print self.e.errno,':',self.e.message
            
    def parse(self,data): #              
        packlen = len(data)        
        inx = 5              
        if data[0:4] <> 'DATA':           
            return 0
            
        while inx < packlen:
            ch = struct.unpack('i8f',data[inx:inx+4*9])
            self.channels[ch[0]] = ch[1:]
            inx += 4*9 #                      
        return inx
                
    def calc(self): #  
        mx = cos(math.radians(self.channels[17][2]))#   heading
        my = sin(math.radians(self.channels[17][2]))
        mz = -0.5 #   .
        pitch = self.channels[17][0]
        roll = self.channels[17][1]
        heading = self.channels[17][2]
        q = ang2q(mx,my,mz)
        d={}
        d['acc_x'] = self.channels[4][5]
        d['acc_y'] = self.channels[4][6]
        d['acc_z'] = self.channels[4][4]
        d['gyro_x'] = self.channels[16][1]
        d['gyro_y'] = self.channels[16][0]
        d['gyro_z'] = self.channels[16][2]
        d['mag_x'] = mx 
        d['mag_y'] = my 
        d['mag_z'] = mz 
        d['lat'] = int(self.channels[20][0] * 10000000)
        d['lng'] = int(self.channels[20][1] * 10000000)
        d['altitude'] = (self.channels[20][2] / 3.28)
        d['speed'] = (self.channels[3][7] * 044.704)	# spped to meters/s
        d['airspeed'] = (self.channels[3][6] * 044.704)	# speed to meters/s
        d['pitch']  = pitch
        d['roll'] =  roll
        d['heading'] = heading
        d['jpitch']  = self.channels[8][0]
        d['jroll'] =  self.channels[8][1]
        d['jrudder'] = self.channels[8][2]
        d['jthrottle'] = self.channels[25][0] #    
        return d
    
    def prn(self):
        d = self.calc()
        print "pitch=",d['pitch'],"roll = ",d['roll'], "heading=", d['heading'],"\n" \
              "lat=",d['lat'],"long=",d['lng'],"altitude=",d['altitude']
        
    def pack(self,ch,data):
        #     XPlane
        res = struct.pack('i8f',ch,data[0],data[1],data[2],data[3],data[4],data[5],data[6],data[7])            
        return res            

###############################     ##############################
import serial      
class AP:
    ser = None
    rxbuffer = ''
    nbyte = 0
    def __init__(self,pt,br,to): # 
        self.connect(pt,br,to)
        rxbuffer = ''
        nbyte = 0        
    def __del__(self): # 
        self.close()
        
    def connect(self,pt,br,to):
        try:
            self.ser = serial.Serial(port=pt, baudrate = br,timeout =to)
            #ser.timeout = None          #block read
            self.ser.timeout = 1            #non-block read
            #ser.timeout = 2              #timeout block read
            self.ser.xonxoff = False     #disable software flow control
            self.ser.rtscts = False     #disable hardware (RTS/CTS) flow control
            self.ser.dsrdtr = False       #disable hardware (DSR/DTR) flow control
        except IOError as self.e:
            print self.e.errno,':', self.e.message
            self.close()
            pass
        
    def close(self):
        try:
           self.ser.close()
        except IOError as self.e:
            print self.e.errno,':',self.e.message
            
    ############################     ##########################
    # 'RAID',(i16 * 3),(i16 * 3),(i16 * 3), (i16),
    # GPS (i32 *2)k, air i16  JOY (i16*4) = 4 + 9 * 2 + 2 + 8 * 2 + 4 * 2 = 42 
    def tx_pack(self,data):
        pack = 'RAID'
        pack += struct.pack('i',data['gyro_x']*100)[:2]
        pack += struct.pack('i',data['gyro_y']*100)[:2]
        pack += struct.pack('i',data['gyro_z']*100)[:2]
        pack += struct.pack('i',data['acc_x']*100)[:2]
        pack += struct.pack('i',data['acc_y']*100)[:2]
        pack += struct.pack('i',data['acc_z']*100)[:2]
        pack += struct.pack('i',data['mag_x']*100)[:2]
        pack += struct.pack('i',data['mag_y']*100)[:2]
        pack += struct.pack('i',data['mag_z']*100)[:2]
        pack += struct.pack('i',data['altitude'])[:2]
        pack += struct.pack('i',data['lat'])        
        pack += struct.pack('i',data['lng'])
        pack += struct.pack('i',data['airspeed']*100)[:2]
        pack += struct.pack('i',data['jroll']*100)[:2]
        pack += struct.pack('i',data['jpitch']*100)[:2]
        pack += struct.pack('i',data['jrudder']*100)[:2]
        pack += struct.pack('i',data['jthrottle']*100)[:2]
        self.send(pack)
        return pack
    
    def send(self,data):
        try:
            self.ser.write(data)                  
        except IOError as self.e:
            print self.e.errno,':', self.e.message
            self.close()
            pass
     
    def rx_pack(self):        
        nchar = self.ser.inWaiting() 
        if nchar:   #                                     
            self.rxbuffer = self.rxbuffer + self.recv(nchar,None)
            lenrx = len(self.rxbuffer)
            if lenrx > 10000: #   
                self.rxbuffer = self.rxbuffer[lenrx-10000:]
        return nchar
                
    def parse(self): #    
        chanels = None
        # 
        strt_char = self.rxbuffer.find('RAI')
        if  strt_char == -1: #  
            self.rxbuffer =''
        else: #  
            self.rxbuffer = self.rxbuffer[strt_char:] #    
            #   
            if self.rxbuffer[3] == 'D': #  
                if len(self.rxbuffer) >= 2*10: #  
                    chanels = struct.unpack('8h',str(bytearray(self.rxbuffer[4:2*10])))                       
                    self.rxbuffer = self.rxbuffer[2*10:] #    
                    
            elif self.rxbuffer[3] == 'S': #   
                if len(self.rxbuffer) >= 6: #                         
                    self.nbyte = ord(self.rxbuffer[4])+ord(self.rxbuffer[5])*256
                    print self.rxbuffer[:4] + str(self.nbyte)
                    if len(self.rxbuffer) >= self.nbyte+6: #                                   
                        confs = 'RAIS' + self.rxbuffer[6:self.nbyte+6]
                        print 'Config recieved CRC= '+hex(ord(confs[self.nbyte+2])+ord(confs[self.nbyte+3])*256)
                        try:
                            f = open('config.rai', 'w')
                            f.write(confs)
                            f.close()      
                        except:
                            print 'Can''t create file ''config.rai'''
                        self.rxbuffer = self.rxbuffer[self.nbyte+6:] #    
                        time.sleep(1)
                
            elif self.rxbuffer[3] == 'R': #   
                self.rxbuffer = self.rxbuffer[4:] #   RAIR 
                try:
                    f = open('config.rai', 'r')
                    confs = f.read()                        
                except:
                    print 'Can''t open file ''config.rai'''
                if f:
                    f.close()
                    self.send(confs)
                print 'Transmit config CRC= '+hex(ord(confs[328])+ord(confs[329])*256)
                time.sleep(1)                          
            
        return chanels
              

    def recv(self,numb,to = 0):
        data = None
        try:
            #self.ser.timeout = to
            data = self.ser.read(numb)  
        except IOError as self.e:
            print self.e.errno,':', self.e.message
            self.close()
            pass
        return data

    
#############################################################################        
#                                                              #
#############################################################################
import time
xp_data ={} #  XPlane #
ap_data ={} #   #
t0_xp =0
t0_ap =0
apf= [0.0]*8

try:
    xp = XPlane(rx_port)
    cycle = 10000
    print "Waiting for X-PLANE on port ", rx_port
    while 1:
        data,addr= xp.rx_pack()
        if data:
            host,port = addr
            print  "X-Plane 10:"+str(addr)
            xp.connect(host,tx_port)
            break
    pass
        
    ap = AP(ComPort,BaudRate,0)
               
    t = time.time() 
    
    while 1:        
        #receive data and sender's address        
        data,addr= xp.rx_pack()
        if data:
            t0_xp += 1
            address = (addr,tx_port)            
            xp.parse(data)
            xp_data = xp.calc()
            a =ap.tx_pack(xp_data)
            pass
       
        ap.rx_pack()
        while len(ap.rxbuffer)>19:
            ap_data = ap.parse()
            if ap_data:
                t0_ap += 1
                for c in [0,1,2,3,4,5,6,7]:
                    apf[c] = (ap_data[c]-1500.0)/500
                #print ap_data #apf
                adata = xp.pack(8,apf)
                adata += xp.pack(25,[apf[3]]*8) #  
                xp.tx_pack(adata)
            else:
                break
        
        if time.time()- t> 1:
            print "cycle xp:"+ str(t0_xp) + " ap: " + str(t0_ap)#+" len aprx:" + str(len(ap.rxbuffer))
            t = time.time()
            t0_ap = 0
            t0_xp = 0
        cycle -=1
        
    xp.close()
    ap.close()
except Exception, e1:
    print "error main...: " + str(e1)
    xp.close()
    ap.close()
